'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _rules = require('../constants/rules');

var _rules2 = _interopRequireDefault(_rules);

var _includes = require('lodash/includes');

var _includes2 = _interopRequireDefault(_includes);

var _isReactComponent = require('../utils/is-react-component');

var _isReactComponent2 = _interopRequireDefault(_isReactComponent);

var _typeOf = require('type-of');

var _typeOf2 = _interopRequireDefault(_typeOf);

var _memoize = require('../utils/memoize');

var _memoize2 = _interopRequireDefault(_memoize);

var _immutable = require('immutable');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Checks that the schema can perform, ordered by performance.
 *
 * @type {Object}
 */

var CHECKS = {
  kind: function kind(object, value) {
    if (object.kind != value) return object.kind;
  },
  type: function type(object, value) {
    if (object.type != value) return object.type;
  },
  isVoid: function isVoid(object, value) {
    if (object.isVoid != value) return object.isVoid;
  },
  minChildren: function minChildren(object, value) {
    if (object.nodes.size < value) return object.nodes.size;
  },
  maxChildren: function maxChildren(object, value) {
    if (object.nodes.size > value) return object.nodes.size;
  },
  kinds: function kinds(object, value) {
    if (!(0, _includes2.default)(value, object.kind)) return object.kind;
  },
  types: function types(object, value) {
    if (!(0, _includes2.default)(value, object.type)) return object.type;
  },
  minLength: function minLength(object, value) {
    var length = object.length;

    if (length < value) return length;
  },
  maxLength: function maxLength(object, value) {
    var length = object.length;

    if (length > value) return length;
  },
  text: function text(object, value) {
    var text = object.text;

    switch ((0, _typeOf2.default)(value)) {
      case 'function':
        if (value(text)) return text;
      case 'regexp':
        if (!text.match(value)) return text;
      default:
        if (text != value) return text;
    }
  },
  anyOf: function anyOf(object, value) {
    var nodes = object.nodes;

    if (!nodes) return;
    var invalids = nodes.filterNot(function (child) {
      return value.some(function (match) {
        return match(child);
      });
    });

    if (invalids.size) return invalids;
  },
  noneOf: function noneOf(object, value) {
    var nodes = object.nodes;

    if (!nodes) return;
    var invalids = nodes.filterNot(function (child) {
      return value.every(function (match) {
        return !match(child);
      });
    });

    if (invalids.size) return invalids;
  },
  exactlyOf: function exactlyOf(object, value) {
    var nodes = object.nodes;

    if (!nodes) return;
    if (nodes.size != value.length) return nodes;

    var invalids = nodes.filterNot(function (child, i) {
      var match = value[i];
      if (!match) return false;
      return match(child);
    });

    if (invalids.size) return invalids;
  }
};

/**
 * Default properties.
 *
 * @type {Object}
 */

var DEFAULTS = {
  rules: []
};

/**
 * Schema.
 *
 * @type {Record}
 */

var Schema = function (_ref) {
  _inherits(Schema, _ref);

  function Schema() {
    _classCallCheck(this, Schema);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(Schema).apply(this, arguments));
  }

  _createClass(Schema, [{
    key: '__getComponent',


    /**
     * Return the component for an `object`.
     *
     * This method is private, because it should always be called on one of the
     * often-changing immutable objects instead, since it will be memoized for
     * much better performance.
     *
     * @param {Mixed} object
     * @return {Component || Void}
     */

    value: function __getComponent(object) {
      var match = this.rules.find(function (rule) {
        return rule.match(object) && rule.component;
      });
      if (!match) return;
      return match.component;
    }

    /**
     * Validate an `object` against the schema, returning the failing rule and
     * reason if the object is invalid, or void if it's valid.
     *
     * This method is private, because it should always be called on one of the
     * often-changing immutable objects instead, since it will be memoized for
     * much better performance.
     *
     * @param {Mixed} object
     * @return {Object || Void}
     */

  }, {
    key: '__validate',
    value: function __validate(object) {
      var reason = void 0;

      var match = this.rules.find(function (rule) {
        if (!rule.match(object)) return;
        debugger;
        if (!rule.validate) return;
        debugger;
        reason = rule.validate(object);
        return reason;
      });

      if (!reason) return;

      return {
        rule: match,
        reason: reason
      };
    }
  }, {
    key: 'kind',


    /**
     * Get the kind.
     *
     * @return {String} kind
     */

    get: function get() {
      return 'schema';
    }
  }], [{
    key: 'create',


    /**
     * Create a new `Schema` with `properties`.
     *
     * @param {Object} properties
     * @return {Schema} mark
     */

    value: function create() {
      var properties = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      if (properties instanceof Schema) return properties;
      return new Schema(normalizeProperties(properties));
    }
  }]);

  return Schema;
}(new _immutable.Record(DEFAULTS));

/**
 * Normalize the `properties` of a schema.
 *
 * @param {Object} properties
 * @return {Object}
 */

function normalizeProperties(properties) {
  var rules = properties.rules;
  var nodes = properties.nodes;
  var marks = properties.marks;

  if (!rules) rules = [];

  // If there's a `nodes` property, it's the node rule shorthand dictionary.
  if (nodes) {
    for (var key in nodes) {
      var value = nodes[key];
      var rule = void 0;

      if ((0, _isReactComponent2.default)(value)) {
        rule.match = { type: key };
        rule.component = value;
      } else {
        rule = _extends({
          kinds: ['block', 'inline'],
          type: key
        }, value);
      }

      rules.push(rule);
    }
  }

  // If there's a `marks` property, it's the mark rule shorthand dictionary.
  if (marks) {
    for (var _key in marks) {
      var _value = marks[_key];
      var _rule = void 0;

      if (_rule.match) {
        _rule = _extends({
          kind: 'mark',
          type: _key
        }, _value);
      } else {
        _rule.match = { type: _key };
        _rule.component = _value;
      }

      rules.push(_rule);
    }
  }

  return {
    rules: rules.map(normalizeRule)
  };
}

/**
 * Normalize a `rule` object.
 *
 * @param {Object} rule
 * @return {Object}
 */

function normalizeRule(rule) {
  return {
    match: normalizeMatch(rule.match),
    validate: normalizeValidate(rule.validate),
    transform: normalizeTransform(rule.transform)
  };
}

/**
 * Normalize a `match` spec.
 *
 * @param {Function || Object || String} match
 * @return {Function}
 */

function normalizeMatch(match) {
  switch ((0, _typeOf2.default)(match)) {
    case 'function':
      return match;
    case 'boolean':
      return function () {
        return match;
      };
    case 'object':
      return normalizeSpec(match);
    default:
      {
        throw new Error('Invalid `match` spec: "' + match + '".');
      }
  }
}

/**
 * Normalize a `validate` spec.
 *
 * @param {Function || Object || String} validate
 * @return {Function}
 */

function normalizeValidate(validate) {
  switch ((0, _typeOf2.default)(validate)) {
    case 'function':
      return validate;
    case 'boolean':
      return function () {
        return validate;
      };
    case 'object':
      return normalizeSpec(validate, true);
  }
}

/**
 * Normalize a `transform` spec.
 *
 * @param {Function || Object || String} transform
 * @return {Function}
 */

function normalizeTransform(transform) {
  switch ((0, _typeOf2.default)(transform)) {
    case 'function':
      return transform;
  }
}

/**
 * Normalize a `spec` object.
 *
 * @param {Object} obj
 * @param {Boolean} giveReason
 * @return {Boolean}
 */

function normalizeSpec(obj, giveReason) {
  var spec = _extends({}, obj);
  if (spec.exactlyOf) spec.exactlyOf = spec.exactlyOf.map(normalizeSpec);
  if (spec.anyOf) spec.anyOf = spec.anyOf.map(normalizeSpec);
  if (spec.noneOf) spec.noneOf = spec.noneOf.map(normalizeSpec);

  return function (node) {
    for (var key in CHECKS) {
      var value = spec[key];
      debugger;

      if (value == null) continue;

      debugger;
      var fail = CHECKS[key];
      var failure = fail(node, value);

      if (!giveReason) {
        return !failure;
      }

      if (failure != undefined) {
        return {
          type: key,
          value: failure
        };
      }
    }
  };
}

/**
 * Export.
 *
 * @type {Record}
 */

exports.default = Schema;